@@ -25,10 +25,11 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Libgfortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 #include "config.h"
+#include <assert.h>
 #include <string.h>
 #include <ctype.h>
 #include <float.h>
@@ -37,7 +38,6 @@ Boston, MA 02111-1307, USA.  */
 #include "libgfortran.h"
 #include "io.h"
 
-
 #define star_fill(p, n) memset(p, '*', n)
 
 
@@ -46,17 +46,15 @@ typedef enum
 sign_t;
 
 
-static int no_leading_blank = 0 ;
-
 void
-write_a (fnode * f, const char *source, int len)
+write_a (st_parameter_dt *dtp, const fnode *f, const char *source, int len)
 {
   int wlen;
   char *p;
 
   wlen = f->u.string.length < 0 ? len : f->u.string.length;
 
-  p = write_block (wlen);
+  p = write_block (dtp, wlen);
   if (p == NULL)
     return;
 
@@ -69,10 +67,10 @@ write_a (fnode * f, const char *source, int len)
     }
 }
 
-static int64_t
+static GFC_INTEGER_LARGEST
 extract_int (const void *p, int len)
 {
-  int64_t i = 0;
+  GFC_INTEGER_LARGEST i = 0;
 
   if (p == NULL)
     return i;
@@ -81,64 +79,144 @@ extract_int (const void *p, int len)
     {
     case 1:
       {
-	int8_t tmp;
+	GFC_INTEGER_1 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
     case 2:
       {
-	int16_t tmp;
+	GFC_INTEGER_2 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
     case 4:
       {
-	int32_t tmp;
+	GFC_INTEGER_4 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
     case 8:
       {
-	int64_t tmp;
+	GFC_INTEGER_8 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
+#ifdef HAVE_GFC_INTEGER_16
+    case 16:
+      {
+	GFC_INTEGER_16 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = tmp;
+      }
+      break;
+#endif
     default:
-      internal_error ("bad integer kind");
+      internal_error (NULL, "bad integer kind");
     }
 
   return i;
 }
 
-static double
+static GFC_UINTEGER_LARGEST
+extract_uint (const void *p, int len)
+{
+  GFC_UINTEGER_LARGEST i = 0;
+
+  if (p == NULL)
+    return i;
+
+  switch (len)
+    {
+    case 1:
+      {
+	GFC_INTEGER_1 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = (GFC_UINTEGER_1) tmp;
+      }
+      break;
+    case 2:
+      {
+	GFC_INTEGER_2 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = (GFC_UINTEGER_2) tmp;
+      }
+      break;
+    case 4:
+      {
+	GFC_INTEGER_4 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = (GFC_UINTEGER_4) tmp;
+      }
+      break;
+    case 8:
+      {
+	GFC_INTEGER_8 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = (GFC_UINTEGER_8) tmp;
+      }
+      break;
+#ifdef HAVE_GFC_INTEGER_16
+    case 16:
+      {
+	GFC_INTEGER_16 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = (GFC_UINTEGER_16) tmp;
+      }
+      break;
+#endif
+    default:
+      internal_error (NULL, "bad integer kind");
+    }
+
+  return i;
+}
+
+static GFC_REAL_LARGEST
 extract_real (const void *p, int len)
 {
-  double i = 0.0;
+  GFC_REAL_LARGEST i = 0;
   switch (len)
     {
     case 4:
       {
-	float tmp;
+	GFC_REAL_4 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
     case 8:
       {
-	double tmp;
+	GFC_REAL_8 tmp;
 	memcpy ((void *) &tmp, p, len);
 	i = tmp;
       }
       break;
+#ifdef HAVE_GFC_REAL_10
+    case 10:
+      {
+	GFC_REAL_10 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = tmp;
+      }
+      break;
+#endif
+#ifdef HAVE_GFC_REAL_16
+    case 16:
+      {
+	GFC_REAL_16 tmp;
+	memcpy ((void *) &tmp, p, len);
+	i = tmp;
+      }
+      break;
+#endif
     default:
-      internal_error ("bad real kind");
+      internal_error (NULL, "bad real kind");
     }
   return i;
-
 }
 
 
@@ -146,14 +224,14 @@ extract_real (const void *p, int len)
    sign_t that gives the sign that we need to produce.  */
 
 static sign_t
-calculate_sign (int negative_flag)
+calculate_sign (st_parameter_dt *dtp, int negative_flag)
 {
   sign_t s = SIGN_NONE;
 
   if (negative_flag)
     s = SIGN_MINUS;
   else
-    switch (g.sign_status)
+    switch (dtp->u.p.sign_status)
       {
       case SIGN_SP:
 	s = SIGN_PLUS;
@@ -172,11 +250,11 @@ calculate_sign (int negative_flag)
 
 /* Returns the value of 10**d.  */
 
-static double
+static GFC_REAL_LARGEST
 calculate_exp (int d)
 {
   int i;
-  double r = 1.0;
+  GFC_REAL_LARGEST r = 1.0;
 
   for (i = 0; i< (d >= 0 ? d : -d); i++)
     r *= 10;
@@ -205,13 +283,14 @@ calculate_exp (int d)
           for Gw.dEe, n' ' means e+2 blanks  */
 
 static fnode *
-calculate_G_format (fnode *f, double value, int len, int *num_blank)
+calculate_G_format (st_parameter_dt *dtp, const fnode *f,
+		    GFC_REAL_LARGEST value, int *num_blank)
 {
   int e = f->u.real.e;
   int d = f->u.real.d;
   int w = f->u.real.w;
   fnode *newf;
-  double m, exp_d;
+  GFC_REAL_LARGEST m, exp_d;
   int low, high, mid;
   int ubound, lbound;
 
@@ -223,8 +302,8 @@ calculate_G_format (fnode *f, double value, int len, int *num_blank)
   /* In case of the two data magnitude ranges,
      generate E editing, Ew.d[Ee].  */
   exp_d = calculate_exp (d);
-  if ((m > 0.0 && m < 0.1 - 0.05 / (double) exp_d)
-      || (m >= (double) exp_d - 0.5 ))
+  if ((m > 0.0 && m < 0.1 - 0.05 / exp_d) || (m >= exp_d - 0.5 ) ||
+      ((m == 0.0) && !(compile_options.allow_std & GFC_STD_F2003)))
     {
       newf->format = FMT_E;
       newf->u.real.w = w;
@@ -243,7 +322,7 @@ calculate_G_format (fnode *f, double value, int len, int *num_blank)
 
   while (low <= high)
     {
-      double temp;
+      GFC_REAL_LARGEST temp;
       mid = (low + high) / 2;
 
       /* 0.1 * 10**mid - 0.5 * 10**(mid-d-1)  */
@@ -287,7 +366,7 @@ calculate_G_format (fnode *f, double value, int len, int *num_blank)
     newf->u.real.d = - (mid - d - 1);
 
   /* For F editing, the scale factor is ignored.  */
-  g.scale_factor = 0;
+  dtp->u.p.scale_factor = 0;
   return newf;
 }
 
@@ -295,9 +374,9 @@ calculate_G_format (fnode *f, double value, int len, int *num_blank)
 /* Output a real number according to its format which is FMT_G free.  */
 
 static void
-output_float (fnode *f, double value, int len)
+output_float (st_parameter_dt *dtp, const fnode *f, GFC_REAL_LARGEST value)
 {
-  /* This must be large enough to accurately hold any value.  */ 
+  /* This must be large enough to accurately hold any value.  */
   char buffer[32];
   char *out;
   char *digits;
@@ -331,12 +410,12 @@ output_float (fnode *f, double value, int len)
 
   /* We should always know the field width and precision.  */
   if (d < 0)
-    internal_error ("Unspecified precision");
+    internal_error (&dtp->common, "Unspecified precision");
 
   /* Use sprintf to print the number in the format +D.DDDDe+ddd
      For an N digit exponent, this gives us (32-6)-N digits after the
      decimal point, plus another one before the decimal point.  */
-  sign = calculate_sign (value < 0.0);
+  sign = calculate_sign (dtp, value < 0.0);
   if (value < 0)
     value = -value;
 
@@ -345,15 +424,19 @@ output_float (fnode *f, double value, int len)
     edigits = 2;
   else
     {
-      abslog = fabs(log10 (value));
+#if defined(HAVE_GFC_REAL_10) || defined(HAVE_GFC_REAL_16)
+      abslog = fabs((double) log10l(value));
+#else
+      abslog = fabs(log10(value));
+#endif
       if (abslog < 100)
 	edigits = 2;
       else
-        edigits = 1 + (int) log10 (abslog);
+        edigits = 1 + (int) log10(abslog);
     }
-  
+
   if (ft == FMT_F || ft == FMT_EN
-      || ((ft == FMT_D || ft == FMT_E) && g.scale_factor != 0))
+      || ((ft == FMT_D || ft == FMT_E) && dtp->u.p.scale_factor != 0))
     {
       /* Always convert at full precision to avoid double rounding.  */
       ndigits = 27 - edigits;
@@ -370,11 +453,28 @@ output_float (fnode *f, double value, int len)
 	ndigits = 27 - edigits;
     }
 
-  sprintf (buffer, "%+-#31.*e", ndigits - 1, value);
-  
+  /* #   The result will always contain a decimal point, even if no
+   *     digits follow it
+   *
+   * -   The converted value is to be left adjusted on the field boundary
+   *
+   * +   A sign (+ or -) always be placed before a number
+   *
+   * 31  minimum field width
+   *
+   * *   (ndigits-1) is used as the precision
+   *
+   *   e format: [-]d.ddde±dd where there is one digit before the
+   *   decimal-point character and the number of digits after it is
+   *   equal to the precision. The exponent always contains at least two
+   *   digits; if the value is zero, the exponent is 00.
+   */
+  sprintf (buffer, "%+-#31.*" GFC_REAL_LARGEST_FORMAT "e",
+           ndigits - 1, value);
+
   /* Check the resulting string has punctuation in the correct places.  */
   if (buffer[2] != '.' || buffer[ndigits + 2] != 'e')
-      internal_error ("printf is broken");
+      internal_error (&dtp->common, "printf is broken");
 
   /* Read the exponent back in.  */
   e = atoi (&buffer[ndigits + 3]) + 1;
@@ -391,7 +491,7 @@ output_float (fnode *f, double value, int len)
   switch (ft)
     {
     case FMT_F:
-      nbefore = e + g.scale_factor;
+      nbefore = e + dtp->u.p.scale_factor;
       if (nbefore < 0)
 	{
 	  nzero = -nbefore;
@@ -411,7 +511,7 @@ output_float (fnode *f, double value, int len)
 
     case FMT_E:
     case FMT_D:
-      i = g.scale_factor;
+      i = dtp->u.p.scale_factor;
       if (value != 0.0)
 	e -= i;
       if (i < 0)
@@ -470,7 +570,7 @@ output_float (fnode *f, double value, int len)
 
     default:
       /* Should never happen.  */
-      internal_error ("Unexpected format token");
+      internal_error (&dtp->common, "Unexpected format token");
     }
 
   /* Round the value.  */
@@ -541,7 +641,7 @@ output_float (fnode *f, double value, int len)
       edigits = 1;
       for (i = abs (e); i >= 10; i /= 10)
 	edigits++;
-      
+
       if (f->u.real.e < 0)
 	{
 	  /* Width not specified.  Must be no more than 3 digits.  */
@@ -571,7 +671,7 @@ output_float (fnode *f, double value, int len)
     w = nbefore + nzero + nafter + (sign != SIGN_NONE ? 2 : 1);
 
   /* Create the ouput buffer.  */
-  out = write_block (w);
+  out = write_block (dtp, w);
   if (out == NULL)
     return;
 
@@ -583,13 +683,13 @@ output_float (fnode *f, double value, int len)
 	break;
     }
   if (i == ndigits)
-    sign = calculate_sign (0);
+    sign = calculate_sign (dtp, 0);
 
   /* Work out how much padding is needed.  */
   nblanks = w - (nbefore + nzero + nafter + edigits + 1);
   if (sign != SIGN_NONE)
     nblanks--;
-  
+
   /* Check the value fits in the specified field width.  */
   if (nblanks < 0 || edigits == -1)
     {
@@ -606,10 +706,10 @@ output_float (fnode *f, double value, int len)
   else
     leadzero = 0;
 
-  /* Padd to full field width.  */
+  /* Pad to full field width.  */
 
 
-  if ( ( nblanks > 0 ) && !no_leading_blank )
+  if ( ( nblanks > 0 ) && !dtp->u.p.no_leading_blank)
     {
       memset (out, ' ', nblanks);
       out += nblanks;
@@ -667,7 +767,7 @@ output_float (fnode *f, double value, int len)
       ndigits -= i;
       out += nafter;
     }
-  
+
   /* Output the exponent.  */
   if (expchar)
     {
@@ -684,22 +784,22 @@ output_float (fnode *f, double value, int len)
       memcpy (out, buffer, edigits);
     }
 
-  if ( no_leading_blank )
+  if (dtp->u.p.no_leading_blank)
     {
       out += edigits;
       memset( out , ' ' , nblanks );
-      no_leading_blank = 0;
+      dtp->u.p.no_leading_blank = 0;
     }
 }
 
 
 void
-write_l (fnode * f, char *source, int len)
+write_l (st_parameter_dt *dtp, const fnode *f, char *source, int len)
 {
   char *p;
-  int64_t n;
+  GFC_INTEGER_LARGEST n;
 
-  p = write_block (f->u.w);
+  p = write_block (dtp, f->u.w);
   if (p == NULL)
     return;
 
@@ -711,9 +811,9 @@ write_l (fnode * f, char *source, int len)
 /* Output a real number according to its format.  */
 
 static void
-write_float (fnode *f, const char *source, int len)
+write_float (st_parameter_dt *dtp, const fnode *f, const char *source, int len)
 {
-  double n;
+  GFC_REAL_LARGEST n;
   int nb =0, res, save_scale_factor;
   char * p, fin;
   fnode *f2 = NULL;
@@ -722,7 +822,7 @@ write_float (fnode *f, const char *source, int len)
 
   if (f->format != FMT_B && f->format != FMT_O && f->format != FMT_Z)
     {
-      res = isfinite (n);
+      res = isfinite (n); 
       if (res == 0)
 	{
 	  nb =  f->u.real.w;
@@ -731,7 +831,9 @@ write_float (fnode *f, const char *source, int len)
 	     not zero.  4 is chosen to allow output of '-Inf' or '+Inf' */
 	     
 	  if (nb == 0) nb = 4;
-	  p = write_block (nb);
+	  p = write_block (dtp, nb);
+          if (p == NULL)
+            return;
 	  if (nb < 3)
 	    {
 	      memset (p, '*',nb);
@@ -788,21 +890,21 @@ write_float (fnode *f, const char *source, int len)
     }
 
   if (f->format != FMT_G)
-    {
-      output_float (f, n, len);
-    }
+    output_float (dtp, f, n);
   else
     {
-      save_scale_factor = g.scale_factor;
-      f2 = calculate_G_format(f, n, len, &nb);
-      output_float (f2, n, len);
-      g.scale_factor = save_scale_factor;
+      save_scale_factor = dtp->u.p.scale_factor;
+      f2 = calculate_G_format (dtp, f, n, &nb);
+      output_float (dtp, f2, n);
+      dtp->u.p.scale_factor = save_scale_factor;
       if (f2 != NULL)
         free_mem(f2);
 
       if (nb > 0)
         {
-          p = write_block (nb);
+	  p = write_block (dtp, nb);
+          if (p == NULL)
+            return;
           memset (p, ' ', nb);
         }
     }
@@ -810,17 +912,19 @@ write_float (fnode *f, const char *source, int len)
 
 
 static void
-write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
+write_int (st_parameter_dt *dtp, const fnode *f, const char *source, int len,
+           const char *(*conv) (GFC_UINTEGER_LARGEST, char *, size_t))
 {
-  uint32_t ns =0;
-  uint64_t n = 0;
+  GFC_UINTEGER_LARGEST n = 0;
   int w, m, digits, nzero, nblank;
-  char *p, *q;
+  char *p;
+  const char *q;
+  char itoa_buf[GFC_BTOA_BUF_SIZE];
 
   w = f->u.integer.w;
   m = f->u.integer.m;
 
-  n = extract_int (source, len);
+  n = extract_uint (source, len);
 
   /* Special case:  */
 
@@ -829,7 +933,7 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
       if (w == 0)
         w = 1;
 
-      p = write_block (w);
+      p = write_block (dtp, w);
       if (p == NULL)
         return;
 
@@ -837,15 +941,7 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
       goto done;
     }
 
-
-  if (len < 8)
-     {
-       ns = n;
-       q = conv (ns);
-     }
-  else
-      q = conv (n);
-
+  q = conv (n, itoa_buf, sizeof (itoa_buf));
   digits = strlen (q);
 
   /* Select a width if none was specified.  The idea here is to always
@@ -854,7 +950,7 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
   if (w == 0)
     w = ((digits < m) ? m : digits);
 
-  p = write_block (w);
+  p = write_block (dtp, w);
   if (p == NULL)
     return;
 
@@ -873,13 +969,13 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
     }
 
 
-  if (!no_leading_blank)
+  if (!dtp->u.p.no_leading_blank)
     {
-  memset (p, ' ', nblank);
-  p += nblank;
-  memset (p, '0', nzero);
-  p += nzero;
-  memcpy (p, q, digits);
+      memset (p, ' ', nblank);
+      p += nblank;
+      memset (p, '0', nzero);
+      p += nzero;
+      memcpy (p, q, digits);
     }
   else
     {
@@ -888,7 +984,7 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
       memcpy (p, q, digits);
       p += digits;
       memset (p, ' ', nblank);
-      no_leading_blank = 0;
+      dtp->u.p.no_leading_blank = 0;
     }
 
  done:
@@ -896,12 +992,16 @@ write_int (fnode *f, const char *source, int len, char *(*conv) (uint64_t))
 }
 
 static void
-write_decimal (fnode *f, const char *source, int len, char *(*conv) (int64_t))
+write_decimal (st_parameter_dt *dtp, const fnode *f, const char *source,
+	       int len,
+               const char *(*conv) (GFC_INTEGER_LARGEST, char *, size_t))
 {
-  int64_t n = 0;
+  GFC_INTEGER_LARGEST n = 0;
   int w, m, digits, nsign, nzero, nblank;
-  char *p, *q;
+  char *p;
+  const char *q;
   sign_t sign;
+  char itoa_buf[GFC_BTOA_BUF_SIZE];
 
   w = f->u.integer.w;
   m = f->u.integer.m;
@@ -915,7 +1015,7 @@ write_decimal (fnode *f, const char *source, int len, char *(*conv) (int64_t))
       if (w == 0)
         w = 1;
 
-      p = write_block (w);
+      p = write_block (dtp, w);
       if (p == NULL)
         return;
 
@@ -923,12 +1023,12 @@ write_decimal (fnode *f, const char *source, int len, char *(*conv) (int64_t))
       goto done;
     }
 
-  sign = calculate_sign (n < 0);
+  sign = calculate_sign (dtp, n < 0);
   if (n < 0)
     n = -n;
 
   nsign = sign == SIGN_NONE ? 0 : 1;
-  q = conv (n);
+  q = conv (n, itoa_buf, sizeof (itoa_buf));
 
   digits = strlen (q);
 
@@ -938,7 +1038,7 @@ write_decimal (fnode *f, const char *source, int len, char *(*conv) (int64_t))
   if (w == 0)
     w = ((digits < m) ? m : digits) + nsign;
 
-  p = write_block (w);
+  p = write_block (dtp, w);
   if (p == NULL)
     return;
 
@@ -983,134 +1083,129 @@ write_decimal (fnode *f, const char *source, int len, char *(*conv) (int64_t))
 
 /* Convert unsigned octal to ascii.  */
 
-static char *
-otoa (uint64_t n)
+static const char *
+otoa (GFC_UINTEGER_LARGEST n, char *buffer, size_t len)
 {
   char *p;
 
-  if (n == 0)
-    {
-      scratch[0] = '0';
-      scratch[1] = '\0';
-      return scratch;
-    }
+  assert (len >= GFC_OTOA_BUF_SIZE);
 
-  p = scratch + SCRATCH_SIZE - 1;
-  *p-- = '\0';
+  if (n == 0)
+    return "0";
+
+  p = buffer + GFC_OTOA_BUF_SIZE - 1;
+  *p = '\0';
 
   while (n != 0)
     {
-      *p = '0' + (n & 7);
-      p--;
+      *--p = '0' + (n & 7);
       n >>= 3;
     }
 
-  return ++p;
+  return p;
 }
 
 
 /* Convert unsigned binary to ascii.  */
 
-static char *
-btoa (uint64_t n)
+static const char *
+btoa (GFC_UINTEGER_LARGEST n, char *buffer, size_t len)
 {
   char *p;
 
-  if (n == 0)
-    {
-      scratch[0] = '0';
-      scratch[1] = '\0';
-      return scratch;
-    }
+  assert (len >= GFC_BTOA_BUF_SIZE);
 
-  p = scratch + SCRATCH_SIZE - 1;
-  *p-- = '\0';
+  if (n == 0)
+    return "0";
+
+  p = buffer + GFC_BTOA_BUF_SIZE - 1;
+  *p = '\0';
 
   while (n != 0)
     {
-      *p-- = '0' + (n & 1);
+      *--p = '0' + (n & 1);
       n >>= 1;
     }
 
-  return ++p;
+  return p;
 }
 
 
 void
-write_i (fnode * f, const char *p, int len)
+write_i (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_decimal (f, p, len, (void *) gfc_itoa);
+  write_decimal (dtp, f, p, len, (void *) gfc_itoa);
 }
 
 
 void
-write_b (fnode * f, const char *p, int len)
+write_b (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_int (f, p, len, btoa);
+  write_int (dtp, f, p, len, btoa);
 }
 
 
 void
-write_o (fnode * f, const char *p, int len)
+write_o (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_int (f, p, len, otoa);
+  write_int (dtp, f, p, len, otoa);
 }
 
 void
-write_z (fnode * f, const char *p, int len)
+write_z (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_int (f, p, len, xtoa);
+  write_int (dtp, f, p, len, xtoa);
 }
 
 
 void
-write_d (fnode *f, const char *p, int len)
+write_d (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_float (f, p, len);
+  write_float (dtp, f, p, len);
 }
 
 
 void
-write_e (fnode *f, const char *p, int len)
+write_e (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_float (f, p, len);
+  write_float (dtp, f, p, len);
 }
 
 
 void
-write_f (fnode *f, const char *p, int len)
+write_f (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_float (f, p, len);
+  write_float (dtp, f, p, len);
 }
 
 
 void
-write_en (fnode *f, const char *p, int len)
+write_en (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_float (f, p, len);
+  write_float (dtp, f, p, len);
 }
 
 
 void
-write_es (fnode *f, const char *p, int len)
+write_es (st_parameter_dt *dtp, const fnode *f, const char *p, int len)
 {
-  write_float (f, p, len);
+  write_float (dtp, f, p, len);
 }
 
 
 /* Take care of the X/TR descriptor.  */
 
 void
-write_x (int m, int nspaces)
+write_x (st_parameter_dt *dtp, int len, int nspaces)
 {
   char *p;
 
-  p = write_block (m);
+  p = write_block (dtp, len);
   if (p == NULL)
     return;
 
   if (nspaces > 0)
-    memset ((char*)(p + m - nspaces), ' ', nspaces);
+    memset (&p[len - nspaces], ' ', nspaces);
 }
 
 
@@ -1121,11 +1216,11 @@ write_x (int m, int nspaces)
    something goes wrong.  */
 
 static int
-write_char (char c)
+write_char (st_parameter_dt *dtp, char c)
 {
   char *p;
 
-  p = write_block (1);
+  p = write_block (dtp, 1);
   if (p == NULL)
     return 1;
 
@@ -1138,23 +1233,24 @@ write_char (char c)
 /* Write a list-directed logical value.  */
 
 static void
-write_logical (const char *source, int length)
+write_logical (st_parameter_dt *dtp, const char *source, int length)
 {
-  write_char (extract_int (source, length) ? 'T' : 'F');
+  write_char (dtp, extract_int (source, length) ? 'T' : 'F');
 }
 
 
 /* Write a list-directed integer value.  */
 
 static void
-write_integer (const char *source, int length)
+write_integer (st_parameter_dt *dtp, const char *source, int length)
 {
   char *p;
   const char *q;
   int digits;
   int width;
+  char itoa_buf[GFC_ITOA_BUF_SIZE];
 
-  q = gfc_itoa (extract_int (source, length));
+  q = gfc_itoa (extract_int (source, length), itoa_buf, sizeof (itoa_buf));
 
   switch (length)
     {
@@ -1181,18 +1277,20 @@ write_integer (const char *source, int length)
 
   digits = strlen (q);
 
-  if(width < digits )
-    width = digits ;
-  p = write_block (width) ;
-  if (no_leading_blank)
+  if (width < digits)
+    width = digits;
+  p = write_block (dtp, width);
+  if (p == NULL)
+    return;
+  if (dtp->u.p.no_leading_blank)
     {
       memcpy (p, q, digits);
-      memset(p + digits ,' ', width - digits) ;
+      memset (p + digits, ' ', width - digits);
     }
   else
     {
-  memset(p ,' ', width - digits) ;
-  memcpy (p + width - digits, q, digits);
+      memset (p, ' ', width - digits);
+      memcpy (p + width - digits, q, digits);
     }
 }
 
@@ -1201,12 +1299,12 @@ write_integer (const char *source, int length)
    the strings if the file has been opened in that mode.  */
 
 static void
-write_character (const char *source, int length)
+write_character (st_parameter_dt *dtp, const char *source, int length)
 {
   int i, extra;
   char *p, d;
 
-  switch (current_unit->flags.delim)
+  switch (dtp->u.p.current_unit->flags.delim)
     {
     case DELIM_APOSTROPHE:
       d = '\'';
@@ -1230,7 +1328,7 @@ write_character (const char *source, int length)
 	  extra++;
     }
 
-  p = write_block (length + extra);
+  p = write_block (dtp, length + extra);
   if (p == NULL)
     return;
 
@@ -1253,55 +1351,70 @@ write_character (const char *source, int length)
 
 
 /* Output a real number with default format.
-   This is 1PG14.7E2 for REAL(4) and 1PG23.15E3 for REAL(8).  */
+   This is 1PG14.7E2 for REAL(4), 1PG23.15E3 for REAL(8),
+   1PG24.15E4 for REAL(10) and 1PG40.31E4 for REAL(16).  */
 
 static void
-write_real (const char *source, int length)
+write_real (st_parameter_dt *dtp, const char *source, int length)
 {
   fnode f ;
-  int org_scale = g.scale_factor;
+  int org_scale = dtp->u.p.scale_factor;
   f.format = FMT_G;
-  g.scale_factor = 1;
-  if (length < 8)
+  dtp->u.p.scale_factor = 1;
+  switch (length)
     {
+    case 4:
       f.u.real.w = 14;
       f.u.real.d = 7;
       f.u.real.e = 2;
-    }
-  else
-    {
+      break;
+    case 8:
       f.u.real.w = 23;
       f.u.real.d = 15;
       f.u.real.e = 3;
+      break;
+    case 10:
+      f.u.real.w = 28;
+      f.u.real.d = 19;
+      f.u.real.e = 4;
+      break;
+    case 16:
+      f.u.real.w = 40;
+      f.u.real.d = 31;
+      f.u.real.e = 4;
+      break;
+    default:
+      internal_error (&dtp->common, "bad real kind");
+      break;
     }
-  write_float (&f, source , length);
-  g.scale_factor = org_scale;
+  write_float (dtp, &f, source , length);
+  dtp->u.p.scale_factor = org_scale;
 }
 
 
 static void
-write_complex (const char *source, int len)
+write_complex (st_parameter_dt *dtp, const char *source, int kind, size_t size)
 {
-  if (write_char ('('))
+  if (write_char (dtp, '('))
     return;
-  write_real (source, len);
+  write_real (dtp, source, kind);
 
-  if (write_char (','))
+  if (write_char (dtp, ','))
     return;
-  write_real (source + len, len);
+  write_real (dtp, source + size / 2, kind);
 
-  write_char (')');
+  write_char (dtp, ')');
 }
 
 
 /* Write the separator between items.  */
 
 static void
-write_separator (void)
+write_separator (st_parameter_dt *dtp)
 {
   char *p;
 
-  p = write_block (options.separator_len);
+  p = write_block (dtp, options.separator_len);
   if (p == NULL)
     return;
 
@@ -1313,49 +1426,65 @@ write_separator (void)
    TODO: handle skipping to the next record correctly, particularly
    with strings.  */
 
-void
-list_formatted_write (bt type, void *p, int len)
+static void
+list_formatted_write_scalar (st_parameter_dt *dtp, bt type, void *p, int kind,
+			     size_t size)
 {
-  static int char_flag;
-
-  if (current_unit == NULL)
+  if (dtp->u.p.current_unit == NULL)
     return;
 
-  if (g.first_item)
+  if (dtp->u.p.first_item)
     {
-      g.first_item = 0;
-      char_flag = 0;
-      write_char (' ');
+      dtp->u.p.first_item = 0;
+      write_char (dtp, ' ');
     }
   else
     {
-      if (type != BT_CHARACTER || !char_flag ||
-	  current_unit->flags.delim != DELIM_NONE)
-	write_separator ();
+      if (type != BT_CHARACTER || !dtp->u.p.char_flag ||
+	  dtp->u.p.current_unit->flags.delim != DELIM_NONE)
+	write_separator (dtp);
     }
 
   switch (type)
     {
     case BT_INTEGER:
-      write_integer (p, len);
+      write_integer (dtp, p, kind);
       break;
     case BT_LOGICAL:
-      write_logical (p, len);
+      write_logical (dtp, p, kind);
       break;
     case BT_CHARACTER:
-      write_character (p, len);
+      write_character (dtp, p, kind);
       break;
     case BT_REAL:
-      write_real (p, len);
+      write_real (dtp, p, kind);
       break;
     case BT_COMPLEX:
-      write_complex (p, len);
+      write_complex (dtp, p, kind, size);
       break;
     default:
-      internal_error ("list_formatted_write(): Bad type");
+      internal_error (&dtp->common, "list_formatted_write(): Bad type");
     }
 
-  char_flag = (type == BT_CHARACTER);
+  dtp->u.p.char_flag = (type == BT_CHARACTER);
+}
+
+
+void
+list_formatted_write (st_parameter_dt *dtp, bt type, void *p, int kind,
+		      size_t size, size_t nelems)
+{
+  size_t elem;
+  char *tmp;
+
+  tmp = (char *) p;
+
+  /* Big loop over all the elements.  */
+  for (elem = 0; elem < nelems; elem++)
+    {
+      dtp->u.p.item_count++;
+      list_formatted_write_scalar (dtp, type, tmp + size*elem, kind, size);
+    }
 }
 
 /*			NAMELIST OUTPUT
@@ -1381,12 +1510,8 @@ list_formatted_write (bt type, void *p, int len)
 
 #define NML_DIGITS 20
 
-/* Stores the delimiter to be used for character objects.  */
-
-static char * nml_delim;
-
 static namelist_info *
-nml_write_obj (namelist_info * obj, index_type offset,
+nml_write_obj (st_parameter_dt *dtp, namelist_info * obj, index_type offset,
 	       namelist_info * base, char * base_name)
 {
   int rep_ctr;
@@ -1412,23 +1537,27 @@ nml_write_obj (namelist_info * obj, index_type offset,
 
   if (obj->type != GFC_DTYPE_DERIVED)
     {
-      write_character ("\n ", 2);
+#ifdef HAVE_CRLF
+      write_character (dtp, "\r\n ", 3);
+#else
+      write_character (dtp, "\n ", 2);
+#endif
       len = 0;
       if (base)
 	{
 	  len =strlen (base->var_name);
-	  for (dim_i = 0; dim_i < strlen (base_name); dim_i++)
+	  for (dim_i = 0; dim_i < (index_type) strlen (base_name); dim_i++)
             {
 	      cup = toupper (base_name[dim_i]);
-	      write_character (&cup, 1);
+	      write_character (dtp, &cup, 1);
             }
 	}
-      for (dim_i =len; dim_i < strlen (obj->var_name); dim_i++)
+      for (dim_i =len; dim_i < (index_type) strlen (obj->var_name); dim_i++)
 	{
 	  cup = toupper (obj->var_name[dim_i]);
-	  write_character (&cup, 1);
+	  write_character (dtp, &cup, 1);
 	}
-      write_character ("=", 1);
+      write_character (dtp, "=", 1);
     }
 
   /* Counts the number of data output on a line, including names.  */
@@ -1436,11 +1565,26 @@ nml_write_obj (namelist_info * obj, index_type offset,
   num = 1;
 
   len = obj->len;
-  obj_size = len;
-  if (obj->type == GFC_DTYPE_COMPLEX)
-    obj_size = 2*len;
-  if (obj->type == GFC_DTYPE_CHARACTER)
-    obj_size = obj->string_length;
+
+  switch (obj->type)
+    {
+
+    case GFC_DTYPE_REAL:
+      obj_size = size_from_real_kind (len);
+      break;
+
+    case GFC_DTYPE_COMPLEX:
+      obj_size = size_from_complex_kind (len);
+      break;
+
+    case GFC_DTYPE_CHARACTER:
+      obj_size = obj->string_length;
+      break;
+
+    default:
+      obj_size = len;      
+    }
+
   if (obj->var_rank)
     obj_size = obj->size;
 
@@ -1483,51 +1627,51 @@ nml_write_obj (namelist_info * obj, index_type offset,
 	  if (rep_ctr > 1)
 	    {
 	      st_sprintf(rep_buff, " %d*", rep_ctr);
-	      write_character (rep_buff, strlen (rep_buff));
-	      no_leading_blank = 1;
+	      write_character (dtp, rep_buff, strlen (rep_buff));
+	      dtp->u.p.no_leading_blank = 1;
 	    }
 	  num++;
 
-	  /* Output the data, if an intrinsic type, or recurse into this 
+	  /* Output the data, if an intrinsic type, or recurse into this
 	     routine to treat derived types.  */
 
 	  switch (obj->type)
 	    {
 
 	    case GFC_DTYPE_INTEGER:
-              write_integer (p, len);
+	      write_integer (dtp, p, len);
               break;
 
 	    case GFC_DTYPE_LOGICAL:
-              write_logical (p, len);
+	      write_logical (dtp, p, len);
               break;
 
 	    case GFC_DTYPE_CHARACTER:
-	      if (nml_delim)
-		write_character (nml_delim, 1);
-	      write_character (p, obj->string_length);
-	      if (nml_delim)
-		write_character (nml_delim, 1);
+	      if (dtp->u.p.nml_delim)
+		write_character (dtp, &dtp->u.p.nml_delim, 1);
+	      write_character (dtp, p, obj->string_length);
+	      if (dtp->u.p.nml_delim)
+		write_character (dtp, &dtp->u.p.nml_delim, 1);
               break;
 
 	    case GFC_DTYPE_REAL:
-              write_real (p, len);
+	      write_real (dtp, p, len);
               break;
 
 	    case GFC_DTYPE_COMPLEX:
-	      no_leading_blank = 0;
+	      dtp->u.p.no_leading_blank = 0;
 	      num++;
-              write_complex (p, len);
+              write_complex (dtp, p, len, obj_size);
               break;
 
 	    case GFC_DTYPE_DERIVED:
 
 	      /* To treat a derived type, we need to build two strings:
 		 ext_name = the name, including qualifiers that prepends
-			    component names in the output - passed to 
+			    component names in the output - passed to
 			    nml_write_obj.
 		 obj_name = the derived type name with no qualifiers but %
-			    appended.  This is used to identify the 
+			    appended.  This is used to identify the
 			    components.  */
 
 	      /* First ext_name => get length of all possible components  */
@@ -1548,7 +1692,7 @@ nml_write_obj (namelist_info * obj, index_type offset,
 		{
 		  strcat (ext_name, dim_i ? "" : "(");
 		  clen = strlen (ext_name);
-		  st_sprintf (ext_name + clen, "%d", obj->ls[dim_i].idx);
+		  st_sprintf (ext_name + clen, "%d", (int) obj->ls[dim_i].idx);
 		  strcat (ext_name, (dim_i == obj->var_rank - 1) ? ")" : ",");
 		}
 
@@ -1567,7 +1711,8 @@ nml_write_obj (namelist_info * obj, index_type offset,
 		   cmp && !strncmp (cmp->var_name, obj_name, obj_name_len);
 		   cmp = retval)
 		{
-		  retval = nml_write_obj (cmp, (index_type)(p - obj->mem_pos),
+		  retval = nml_write_obj (dtp, cmp,
+					  (index_type)(p - obj->mem_pos),
 					  obj, ext_name);
 		}
 
@@ -1576,19 +1721,23 @@ nml_write_obj (namelist_info * obj, index_type offset,
 	      goto obj_loop;
 
             default:
-              internal_error ("Bad type for namelist write");
+	      internal_error (&dtp->common, "Bad type for namelist write");
             }
 
 	  /* Reset the leading blank suppression, write a comma and, if 5
 	     values have been output, write a newline and advance to column
 	     2. Reset the repeat counter.  */
 
-	  no_leading_blank = 0;
-	  write_character (",", 1);
+	  dtp->u.p.no_leading_blank = 0;
+	  write_character (dtp, ",", 1);
 	  if (num > 5)
 	    {
 	      num = 0;
-	      write_character ("\n ", 2);
+#ifdef HAVE_CRLF
+	      write_character (dtp, "\r\n ", 3);
+#else
+	      write_character (dtp, "\n ", 2);
+#endif
 	    }
 	  rep_ctr = 1;
 	}
@@ -1616,12 +1765,12 @@ obj_loop:
 }
 
 /* This is the entry function for namelist writes.  It outputs the name
-   of the namelist and iterates through the namelist by calls to 
-   nml_write_obj.  The call below has dummys in the arguments used in 
+   of the namelist and iterates through the namelist by calls to
+   nml_write_obj.  The call below has dummys in the arguments used in
    the treatment of derived types.  */
 
 void
-namelist_write (void)
+namelist_write (st_parameter_dt *dtp)
 {
   namelist_info * t1, *t2, *dummy = NULL;
   index_type i;
@@ -1632,47 +1781,51 @@ namelist_write (void)
 
   /* Set the delimiter for namelist output.  */
 
-  tmp_delim = current_unit->flags.delim;
-  current_unit->flags.delim = DELIM_NONE;
+  tmp_delim = dtp->u.p.current_unit->flags.delim;
+  dtp->u.p.current_unit->flags.delim = DELIM_NONE;
   switch (tmp_delim)
     {
     case (DELIM_QUOTE):
-      nml_delim = "\"";
+      dtp->u.p.nml_delim = '"';
       break;
 
     case (DELIM_APOSTROPHE):
-      nml_delim = "'";
+      dtp->u.p.nml_delim = '\'';
       break;
 
     default:
-      nml_delim = NULL;
+      dtp->u.p.nml_delim = '\0';
+      break;
     }
 
-  write_character ("&",1);
+  write_character (dtp, "&", 1);
 
   /* Write namelist name in upper case - f95 std.  */
 
-  for (i = 0 ;i < ioparm.namelist_name_len ;i++ )
+  for (i = 0 ;i < dtp->namelist_name_len ;i++ )
     {
-      c = toupper (ioparm.namelist_name[i]);
-      write_character (&c ,1);
-	    }
+      c = toupper (dtp->namelist_name[i]);
+      write_character (dtp, &c ,1);
+    }
 
-  if (ionml != NULL)
+  if (dtp->u.p.ionml != NULL)
     {
-      t1 = ionml;
+      t1 = dtp->u.p.ionml;
       while (t1 != NULL)
 	{
 	  t2 = t1;
-	  t1 = nml_write_obj (t2, dummy_offset, dummy, dummy_name);
+	  t1 = nml_write_obj (dtp, t2, dummy_offset, dummy, dummy_name);
 	}
     }
-  write_character ("  /\n", 4);
+#ifdef HAVE_CRLF
+  write_character (dtp, "  /\r\n", 5);
+#else
+  write_character (dtp, "  /\n", 4);
+#endif
 
   /* Recover the original delimiter.  */
 
-  current_unit->flags.delim = tmp_delim;
+  dtp->u.p.current_unit->flags.delim = tmp_delim;
 }
 
 #undef NML_DIGITS
-
